# frozen_string_literal: true

require "openssl"
require "date"
require "json"
require "base64"

require "bluedoc/license/version"
require "bluedoc/license/encryptor"
require "bluedoc/license/boundary"

module BlueDoc
  class License
    class Error < StandardError; end
    class ImportError < Error; end
    class ValidationError < Error; end

    class << self
      attr_reader :encryption_key
      @encryption_key = nil

      def encryption_key=(key)
        if key && !key.is_a?(OpenSSL::PKey::RSA)
          raise ArgumentError, "No RSA encryption key provided."
        end

        @encryption_key = key
        @encryptor = nil
      end

      def encryptor
        @encryptor ||= Encryptor.new(self.encryption_key)
      end

      def import(data)
        if data.nil?
          raise ImportError, "No license data."
        end

        data = Boundary.remove_boundary(data)

        begin
          license_json = encryptor.decrypt(data)
        rescue Encryptor::Error
          raise ImportError, "License data could not be decrypted."
        end

        begin
          attributes = JSON.parse(license_json)
        rescue JSON::ParseError
          raise ImportError, "License data is invalid JSON."
        end

        new(attributes)
      end
    end

    attr_reader :version
    attr_accessor :licensee, :starts_at, :expires_at
    attr_accessor :restrictions, :features

    def initialize(attributes = {})
      load_attributes(attributes)
    end

    def valid?
      return false if !licensee || !licensee.is_a?(Hash) || licensee.length == 0
      return false if !starts_at || !starts_at.is_a?(Date)
      return false if expires_at && !expires_at.is_a?(Date)
      return false if restrictions && !restrictions.is_a?(Hash)
      return false if features && !features.is_a?(Array)

      true
    end

    def validate!
      raise ValidationError, "License is invalid" unless valid?
    end

    def will_expire?
      self.expires_at == nil ? false : true
    end

    def expired?
      will_expire? && Date.today >= self.expires_at
    end

    def restricted?(key = nil)
      if key
        restricted? && restrictions.has_key?(key)
      else
        restrictions && restrictions.length >= 1
      end
    end

    def allow_feature?(key)
      self.features.include?(key.to_s)
    end

    private

      def load_attributes(attributes)
        attributes = Hash[attributes.map { |k, v| [k.to_s, v] }]

        version = attributes["version"] || 1
        unless version && version == 1
          raise ArgumentError, "Version is too new"
        end

        @version = version

        @licensee = attributes["licensee"]
        @features = attributes["features"] || []

        %w(starts_at expires_at).each do |attr|
          value = attributes[attr]
          value = Date.parse(value) rescue nil if value.is_a?(String)

          next unless value

          send("#{attr}=", value)
        end

        restrictions = attributes["restrictions"]
        if restrictions && restrictions.is_a?(Hash)
          restrictions = Hash[restrictions.map { |k, v| [k.to_sym, v] }]
          @restrictions = restrictions
        end
      end
  end
end
